@@ -85,6 +85,36 @@ def _typeof(p: inspect.Parameter):
         str  # type: ignore
 
 
+class _Pool:
+    """Thread-safe pool of something, created on demand."""
+
+    def __init__(self, fun: Callable[[int], Any]):
+        self._lock = threading.Lock()
+        self._fun = fun
+        self._nobjs = 0
+        self._available: Set[Any] = set()
+        self._using: Set[Any] = set()
+
+    def get(self):
+        """Get a object from the pool, possibly creating one if needed."""
+        with self._lock:
+            try:
+                obj = self._available.pop()
+            except KeyError:
+                log.debug(f"creating new obj with {self._fun}")
+                obj = self._fun(self._nobjs)
+                self._nobjs += 1
+            finally:
+                self._using.add(obj)
+            return obj
+
+    def ret(self, obj):
+        """Return object to pool."""
+        with self._lock:
+           self._using.remove(obj)
+           self._available.add(obj)
+
+
 class Reference:
     """Convenient object wrapper class.
 
@@ -105,13 +135,15 @@ class Reference:
     class Local(object):
         pass
 
-    def __init__(self, obj: Any = None, set_name: str = "set", fun: Optional[Callable] = None):
+    def __init__(self, obj: Any = None, set_name: str = "set", fun: Optional[Callable] = None, pool: bool = False):
         """Constructor parameters:
 
         - set_name: provide another prefix for the "set" functions.
         - obj: object to be wrapped, can also be provided later.
         - fun: function to generated a per-thread wrapped object.
+        - pool: whether to use a pool on top of fun.
         """
+        self._pool = pool
         self._set(obj=obj, fun=fun, mandatory=False)
         if set_name and set_name != "_set":
             setattr(self, set_name, self._set)
@@ -122,7 +154,7 @@ class Reference:
         """Set current wrapped object."""
         log.debug(f"setting reference to {obj} ({type(obj)})")
         self._fun = None
-        self._nthreads = 1
+        self._nobjs = 1
         self._local = self.Local()
         self._local.obj = obj
         return self
@@ -130,7 +162,9 @@ class Reference:
     def _set_fun(self, fun: Callable[[int], Any]):
         """Set current wrapped object generation function."""
         self._fun = fun
-        self._nthreads = 0
+        if self._pool:
+            self._pool_set = _Pool(fun)
+        self._nobjs = 0
         self._local = threading.local()
         return self
 
@@ -148,10 +182,20 @@ class Reference:
     def _get_obj(self):
         """Get current wrapped object."""
         if self._fun and not hasattr(self._local, "obj"):
-            self._local.obj = self._fun(self._nthreads)
-            self._nthreads += 1
+            if self._pool:
+                self._local.obj = self._pool_set.get()
+                self._nobjs = self._pool_set._nobjs  # FIXME
+            else:
+                self._local.obj = self._fun(self._nobjs)
+                self._nobjs += 1
         return self._local.obj
 
+    def _ret_obj(self):
+        """Return current wrapped object to internal pool."""
+        assert self._pool
+        self._pool_set.ret(self._local.obj)
+        delattr(self._local, "obj")
+
     def __getattr__(self, item):
         """Forward everything unknown to contained object."""
         return self._get_obj().__getattribute__(item)
@@ -585,10 +629,10 @@ class FlaskSimpleAuth:
             realm = "".join(c if keep_char(c) else "-" for c in realm)
             realm = "-".join(filter(lambda s: s != "", realm.split("-")))
         self._realm = realm
-        # token expiration
+        # token expiration in minutes
         self._delay = conf.get("FSA_TOKEN_DELAY", 60.0)
         self._grace = conf.get("FSA_TOKEN_GRACE", 0.0)
-        self._renewal = conf.get("FSA_TOKEN_RENEWAL", 0.25)  # only for cookies
+        self._renewal = conf.get("FSA_TOKEN_RENEWAL", 0.25)  # ratio, only for cookies
         # token signature
         if "FSA_TOKEN_SECRET" in conf:
             self._secret = conf["FSA_TOKEN_SECRET"]
