@@ -7,7 +7,7 @@ from anodb import DB  # type: ignore
 from FlaskSimpleAuth import Reference, Flask, Response  # type: ignore
 
 # this reference will behave as a DB
-db : Any = Reference()
+db: Any = Reference()
 
 
 # always close current transaction
@@ -22,5 +22,5 @@ def db_commit(res: Response):
 # module initialization
 def init_app(app: Flask):
     cf = app.config
-    db.set(DB(cf["DB_TYPE"], cf["DB_CONN"], cf["DB_SQL"], cf["DB_OPTIONS"]))
+    db.set_fun(lambda i: DB(cf["DB_TYPE"], cf["DB_CONN"], cf["DB_SQL"], cf["DB_OPTIONS"]))
     app.after_request(db_commit)
